var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx } from "react/jsx-runtime";
import { isNull, isUndefined } from 'lodash';
import { useState, useMemo, useRef } from 'react';
import { useId } from '@react-devui/hooks';
import { findNested, getClassName } from '@react-devui/utils';
import { useGeneralContext, useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { DFocusVisible } from '../_focus-visible';
import { useFormControl } from '../form';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DPanel } from './Panel';
import { MultipleTreeNode } from './multiple-node';
import { SingleTreeNode } from './single-node';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DTree' });
export function DTree(props) {
    const _a = useComponentConfig(COMPONENT_NAME, props), { dFormControl, dList, dModel, dHeight, dExpands, dExpandAll = false, dShowLine = false, dDisabled = false, dMultiple = false, dOnlyLeafSelectable = true, dCustomItem, onModelChange, onFirstExpand, onExpandsChange } = _a, restProps = __rest(_a, ["dFormControl", "dList", "dModel", "dHeight", "dExpands", "dExpandAll", "dShowLine", "dDisabled", "dMultiple", "dOnlyLeafSelectable", "dCustomItem", "onModelChange", "onFirstExpand", "onExpandsChange"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    const { gDisabled } = useGeneralContext();
    //#endregion
    //#region Ref
    const comboboxKeyDownRef = useRef(null);
    //#endregion
    const dataRef = useRef({
        hasExpandList: new Set(dExpands),
    });
    const uniqueId = useId();
    const getGroupId = (val) => `${dPrefix}tree-group-${val}-${uniqueId}`;
    const getItemId = (val) => `${dPrefix}tree-item-${val}-${uniqueId}`;
    const renderNodes = useMemo(() => dList.map((item) => dMultiple
        ? new MultipleTreeNode(item, (origin) => origin.value, {
            disabled: item.disabled,
        })
        : new SingleTreeNode(item, (origin) => origin.value, {
            disabled: item.disabled,
        })), [dMultiple, dList]);
    const [nodesMap, initExpandAll] = useMemo(() => {
        const nodes = new Map();
        const expandAllNodes = [];
        const reduceArr = (arr) => {
            for (const item of arr) {
                nodes.set(item.id, item);
                if (item.children) {
                    expandAllNodes.push(item.id);
                    reduceArr(item.children);
                }
            }
        };
        reduceArr(renderNodes);
        return [nodes, expandAllNodes];
    }, [renderNodes]);
    const formControlInject = useFormControl(dFormControl);
    const [_select, changeSelect] = useDValue(dMultiple ? [] : null, dModel, (value) => {
        var _a;
        if (onModelChange) {
            if (dMultiple) {
                onModelChange(value, value.map((v) => nodesMap.get(v).origin));
            }
            else {
                onModelChange(value, isNull(value) ? null : (_a = nodesMap.get(value)) === null || _a === void 0 ? void 0 : _a.origin);
            }
        }
    }, undefined, formControlInject);
    const select = useMemo(() => (dMultiple ? new Set(_select) : _select), [_select, dMultiple]);
    renderNodes.forEach((node) => {
        node.updateStatus(select);
    });
    const [_expandIds, changeExpandIds] = useDValue(dExpandAll ? initExpandAll : [], dExpands, (value) => {
        if (onExpandsChange) {
            onExpandsChange(value, value.map((v) => nodesMap.get(v).origin));
        }
    });
    const expandIds = useMemo(() => new Set(_expandIds), [_expandIds]);
    const disabled = dDisabled || gDisabled || (dFormControl === null || dFormControl === void 0 ? void 0 : dFormControl.control.disabled);
    const hasSelected = dMultiple ? select.size > 0 : !isNull(select);
    const [focusVisible, setFocusVisible] = useState(false);
    const [_focusNode, setFocusNode] = useState();
    const focusNode = (() => {
        let node;
        if (_focusNode) {
            node = nodesMap.get(_focusNode.id);
            if (node && node.enabled) {
                return node;
            }
        }
        if (hasSelected) {
            node = findNested(renderNodes, (node) => node.enabled && node.checked);
        }
        if (isUndefined(node)) {
            node = findNested(renderNodes, (node) => node.enabled);
        }
        return node;
    })();
    return (_jsx(DFocusVisible, Object.assign({ onFocusVisibleChange: setFocusVisible }, { children: ({ render: renderFocusVisible }) => {
            var _a;
            return renderFocusVisible(_jsx(DPanel, Object.assign({}, restProps, { ref: comboboxKeyDownRef, className: getClassName(restProps.className, {
                    'is-disabled': disabled,
                }), style: Object.assign(Object.assign({}, restProps.style), { maxHeight: dHeight }), tabIndex: (_a = restProps.tabIndex) !== null && _a !== void 0 ? _a : (disabled ? -1 : 0), onKeyDown: (e) => {
                    var _a, _b, _c, _d, _e, _f, _g, _h;
                    (_a = restProps.onKeyDown) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                    switch (e.code) {
                        case 'ArrowUp':
                            e.preventDefault();
                            (_b = comboboxKeyDownRef.current) === null || _b === void 0 ? void 0 : _b.call(comboboxKeyDownRef, 'prev');
                            break;
                        case 'ArrowDown':
                            e.preventDefault();
                            (_c = comboboxKeyDownRef.current) === null || _c === void 0 ? void 0 : _c.call(comboboxKeyDownRef, 'next');
                            break;
                        case 'ArrowLeft':
                            e.preventDefault();
                            (_d = comboboxKeyDownRef.current) === null || _d === void 0 ? void 0 : _d.call(comboboxKeyDownRef, 'prev-level');
                            break;
                        case 'ArrowRight':
                            e.preventDefault();
                            (_e = comboboxKeyDownRef.current) === null || _e === void 0 ? void 0 : _e.call(comboboxKeyDownRef, 'next-level');
                            break;
                        case 'Home':
                            e.preventDefault();
                            (_f = comboboxKeyDownRef.current) === null || _f === void 0 ? void 0 : _f.call(comboboxKeyDownRef, 'first');
                            break;
                        case 'End':
                            e.preventDefault();
                            (_g = comboboxKeyDownRef.current) === null || _g === void 0 ? void 0 : _g.call(comboboxKeyDownRef, 'last');
                            break;
                        case 'Enter':
                            e.preventDefault();
                            (_h = comboboxKeyDownRef.current) === null || _h === void 0 ? void 0 : _h.call(comboboxKeyDownRef, 'click');
                            break;
                        default:
                            break;
                    }
                }, dGetGroupId: getGroupId, dGetItemId: getItemId, dList: renderNodes, dExpandIds: expandIds, dHeight: dHeight !== null && dHeight !== void 0 ? dHeight : Infinity, dPadding: undefined, dFocusItem: focusNode, dCustomItem: dCustomItem, dShowLine: dShowLine, dMultiple: dMultiple, dFocusVisible: focusVisible, onFocusChange: setFocusNode, onExpandChange: (item) => {
                    const isExpand = expandIds.has(item.id);
                    if (!item.origin.loading) {
                        if (isExpand) {
                            changeExpandIds((draft) => {
                                draft.splice(draft.findIndex((id) => id === item.id), 1);
                            });
                        }
                        else {
                            if (!dataRef.current.hasExpandList.has(item.id)) {
                                dataRef.current.hasExpandList.add(item.id);
                                onFirstExpand === null || onFirstExpand === void 0 ? void 0 : onFirstExpand(item.id, item.origin);
                            }
                            changeExpandIds((draft) => {
                                draft.push(item.id);
                            });
                        }
                    }
                }, onClickItem: (node) => {
                    if (dMultiple) {
                        const checkeds = node.changeStatus(node.checked ? 'UNCHECKED' : 'CHECKED', select);
                        changeSelect(Array.from(checkeds.keys()));
                    }
                    else {
                        if (!dOnlyLeafSelectable || node.isLeaf) {
                            changeSelect(node.id);
                        }
                    }
                } })));
        } })));
}
